
# q-expansion of newform 5929.2.a.y, downloaded from the LMFDB on 26 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5929
weight = 2
poly_data = [-2, -6, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0], 1], [[0, 1, 0], 1], [[-4, -2, 1], 1]]

hecke_ring_character_values = None
aps_data = [[1, -1, 0], [0, 1, 0], [0, 0, 1], [0, 0, 0], [0, 0, 0], [-2, -1, 1], [-2, -1, 1], [0, 2, 2], [2, 0, -1], [0, 2, 2], [4, 1, 0], [-6, 2, 1], [-6, 1, -1], [2, 0, 2], [-2, 1, -1], [-4, 0, -2], [0, -1, 0], [6, -1, 1], [-2, 4, 1], [2, 4, 5], [-6, 1, -1], [10, 0, 2], [-4, 2, -2], [8, -2, 1], [4, 2, 3], [6, 3, 5], [2, -5, -3], [0, 4, 4], [-4, -2, -6], [8, 2, 1], [-8, 4, 4], [8, -2, -2], [-2, -2, 3], [0, 6, -2], [0, 4, 4], [2, 4, -2], [0, -4, -7], [-8, -2, -2], [-12, 0, -4], [-6, 5, 3], [6, -6, -1], [-4, 4, 3], [14, 2, 3], [4, 4, 0], [12, 4, 0], [-6, 7, 1], [0, -4, 4], [-4, -3, 8], [-16, 2, 2], [-4, 0, 3], [12, 6, 2], [-8, 4, 4], [-6, -7, -1], [-12, 1, 2], [10, 4, 0], [-16, 0, 0], [-2, -2, 2], [0, -4, -4], [-8, -4, -4], [4, 6, -6], [0, -10, -2], [-10, 3, 5], [16, -6, 2], [10, -5, -3], [4, 2, -3], [-8, 2, 1], [6, 6, 5], [-8, -2, -2], [6, -8, -2], [-10, 11, 5], [-16, -2, -7], [-10, 4, 2], [0, -5, -2], [12, 0, 4], [-2, -4, 7], [0, 7, 6], [16, 6, 1], [14, -10, -2], [2, 8, -4], [2, -7, -9], [6, 3, 5], [-20, 0, -6], [2, 4, -10], [-12, -2, -5], [-12, 8, 4], [-2, -10, 3], [6, -14, -3], [-4, 0, -4], [18, -3, 3], [2, 0, -3], [8, 3, 8], [-12, -8, -4], [10, 8, 11], [14, 4, 2], [-4, -6, 6], [8, -4, -4], [-12, 0, 3], [-12, 14, -1], [4, 10, 6], [-36, 4, 0], [-8, 0, -8], [-12, 0, 4], [-8, 2, 2], [-8, -6, 2], [-8, 16, 8], [-28, 2, 3], [18, -3, -5], [-10, 15, 1], [8, 6, -6], [-30, 9, -1], [-4, -8, 4], [24, -6, -6], [-16, -4, -2], [-12, -1, 10], [2, -6, 1], [-6, -14, -11], [4, 5, -6], [8, 5, 10], [-4, 10, -9], [-10, 8, -10], [-4, -8, 5], [-12, -2, 2], [-26, -1, -7], [12, -8, -4], [-28, -7, -10], [4, 14, 2], [-12, 2, 7], [24, 3, 6], [28, -7, 0], [14, 11, 13], [0, -8, 8], [-8, -4, -12], [-10, 0, -7], [-10, 4, 0], [-6, 5, -5], [-18, 7, 9], [2, 14, -2], [-20, 10, 6], [32, 0, 1], [28, -4, 0], [20, 10, 6], [0, -4, -4], [-22, 14, 5], [-14, 4, -2], [16, 12, 13], [-8, 15, 2], [6, -1, -7], [-30, -3, -5], [12, -7, -2], [4, -16, -8], [-4, 10, 6], [4, -2, 7], [-4, 8, 12], [4, -8, 12], [-4, 10, -10], [28, -6, -6], [-40, 0, 0], [30, -4, -8], [14, 7, 9], [-18, -9, -7], [-14, 6, 3], [32, 4, 12], [14, -4, 10], [4, -21, -6], [8, 2, 1], [0, -17, -6], [-8, 2, 6], [18, -7, -1], [-12, -8, 4], [24, 4, -1], [-8, 4, 4], [-22, -6, -18], [18, 8, -6], [-12, -2, -6], [24, -3, -10], [0, 2, -1], [-6, 4, 6], [26, 2, 9], [-12, -12, 0], [6, -13, 5], [8, 4, 16], [0, 2, 2], [-4, -2, 17], [-14, -7, 15], [26, 10, 19], [6, 2, 6], [-4, 8, -12], [-18, 7, 9], [4, -2, 18], [-24, -8, 0], [-8, -10, 7], [8, -4, -12], [22, -8, 9], [2, -6, 6], [-26, -4, -14], [16, 10, -7], [16, -26, -10], [34, 10, -7], [-18, -17, -15], [0, 4, -12], [24, 0, 0], [-8, 4, -12], [10, 14, -10], [10, 17, -1], [-8, 21, -8], [20, 0, 17], [36, -5, 0], [24, -8, -8], [20, -12, -8], [18, 9, 15], [16, -2, -10], [32, -8, -9], [22, 6, 5], [-14, -11, -21], [0, 16, 8], [28, -6, 5], [-42, -8, -10], [-10, 3, 5], [8, 18, -2], [4, 10, -5], [-8, 2, -14], [4, -12, -16], [10, -18, 7], [34, 2, 11], [-28, -6, 6], [-20, -6, -2], [6, 0, 12], [16, -4, 8], [-4, 12, -8], [-22, -8, 10], [30, -18, -1], [-28, -10, -6], [8, -24, -8], [8, -24, 0], [4, -7, 14], [-4, -4, 8], [4, 10, -1], [32, -24, 0], [-46, 0, 3], [36, -1, -12], [6, 0, -1], [56, -10, -2], [2, 11, -3], [-22, -6, -5], [-54, 5, 3], [-20, 0, 4], [18, 7, 1], [-2, -1, -7], [-32, 0, 0], [-40, -8, 0], [-4, 20, 16], [18, 5, -5], [10, 0, 13], [50, -16, -4], [-2, 7, -15], [8, -20, -4], [24, -28, -6], [-36, 2, -10], [-20, 16, 5], [-38, -3, -5], [14, -12, -6], [-8, -24, -16], [34, 5, -13], [-14, -11, -21], [2, 8, 12], [10, 17, 23], [-2, -18, -7], [16, -22, -13], [10, 4, 6], [-24, -6, 2], [0, 0, 1], [-18, -16, -1], [-22, -16, 0], [16, 12, 12], [10, -7, -9], [-44, 13, 0], [6, 2, 5], [4, 20, -8], [12, 8, 12], [-44, -6, 6], [8, 12, 4], [10, -16, -15], [-16, 12, 4], [10, 13, 3], [-28, 2, -10], [6, 20, -15], [12, -4, 2], [-48, 16, 0], [-16, -15, 2], [30, 15, 25], [42, 10, 3], [-46, 13, 11], [24, -6, -6], [40, 6, -2], [24, -36, -12], [2, 9, -9], [16, 21, 22], [-36, -8, -1], [4, -18, -22], [20, -6, 14], [38, -9, 9], [44, 10, 14], [42, 0, -6], [24, -18, 6], [22, -12, -13], [-30, 10, -15], [14, -6, -19], [-20, -12, -7], [0, -16, -20], [-24, 8, 0], [12, 20, 8], [-10, 4, 0], [44, -2, 2], [18, 2, -13], [-10, 0, -10], [-10, -9, 17], [60, -13, -2], [-16, -16, 16], [18, 16, 16], [36, 14, 18], [0, -6, 18], [24, -6, -11], [38, -5, 5], [14, -18, 3], [28, 16, 12], [-8, -2, 1], [-2, 0, 19], [-60, -9, -6], [2, -20, 14], [36, -14, -2], [-68, 16, 4], [0, -4, 2], [16, 8, 24], [16, -2, 14], [-6, -12, -2], [4, -7, 14], [46, -2, -1], [-22, -7, 7], [-48, -14, -11], [-32, 28, 4], [4, 2, -3], [26, -14, 5], [-38, 1, 7], [-6, 10, -3], [-40, 10, 15], [32, -24, 0], [8, 24, 15], [30, -16, 7], [24, -24, -16], [8, 20, -1], [-8, 30, 14], [54, 2, 5], [-6, 32, 12], [-8, -8, -8], [16, -28, 12], [10, -23, -9], [-32, -11, -14], [44, -14, 6], [8, 0, 0], [26, -22, -13], [-48, -12, -12], [10, -15, 7], [-42, -14, 5], [2, 4, 6], [8, 13, -16], [14, 14, 23], [-30, -3, -5], [0, -4, 4], [12, 0, -4], [-10, 0, -23], [2, -3, -21], [-24, 0, 0], [-4, -4, 24], [24, 0, 16], [18, -6, -1], [18, -31, -1], [26, -12, 6], [30, -5, -3], [-34, 39, 17], [14, 7, 17], [-6, -22, -18], [-26, 10, -19], [-26, 0, -2], [16, 4, 4], [-24, -2, 14], [-12, -7, -28], [-28, -12, -20], [-16, 0, -3], [32, 20, -4], [-20, 9, 12], [-2, 22, 19], [-4, -28, 3], [-50, 4, -3], [-12, -9, -2], [-18, -6, -27], [-28, 2, 6], [-24, -4, 12], [-38, -8, -4], [12, -2, 18], [-20, -10, 18], [2, 5, -13], [28, 2, 5], [48, -24, -8], [26, 12, 32], [-66, 7, -7], [-4, 16, 12], [-6, -24, 0], [20, -8, -4], [-10, -13, -27], [40, -11, 16], [-10, 15, -15], [-8, 2, -22], [16, -24, -8], [-30, 28, 14], [-52, -14, 6], [0, 0, -7], [16, -8, 8], [40, -13, -2], [-30, 20, 8], [-32, -12, 20], [0, -6, -13], [-10, -9, -15], [72, 2, 14], [84, 6, 2], [10, -16, -12], [44, -8, 3], [22, -4, 3], [-36, 34, 14], [52, 3, 14], [-40, 0, 16]]
